<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\News;
use App\Models\User;
use App\Models\Role;
use Auth;
use Carbon\Carbon;

class NewsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {        
        // Validation ? 
        $user = User::find(1); //Auth::user();
        if (!$user->hasRole(Role::READ, 'News')){
            return response()->json('Acces denied', 401);
        }
        return $user->news;        
    }    

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {               
        $user = User::find(1);
        if (!$user->hasRole(Role::CREATE, 'News')){
            return response()->json('Acces denied', 401);
        }
        $data = [];
        $data['title'] = $request->input('title', null);
        $data['body'] = $request->input('body', null);
        if (!News::isValid(['title' => $data['title']])) {
            return response()->json('Title malformatted', 400);
        }
        if (!News::isValid(['body' => $data['body']])) {
            return response()->json('Body malformatted', 400);
        }
        
        $news = new News($data);                 
        $user->news()->save($news);    
        return $news;
    }
    
    public function search(Request $request)
    {
        return response()->json("Not yet implemented");
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {   
        $user = User::find(1);
        if (!$user->hasRole(Role::READ, 'News')){
            return response()->json('Acces denied', 401);
        }
        // Validation ?
        $new = News::find($id);
        if (!isset($new)) {
            return response()->json('Not found', 404);
        }
        // Owner ?        
        $new->user;
        if ($new->user->id != $user->id) {
            return response()->json('Forbidden', 403);
        }
        return $new;
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {                
        $user = User::find(1);
        if (!$user->hasRole(Role::UPDATE, 'News')){
            return response()->json('Acces denied', 401);
        }
        $news = News::find($id);
        // exists ?
        if (!isset($news)) {
            return response()->json('Not found', 404);
        }
        // Owner ?
        $news->user;
        if ($news->user->id != $user->id) {
            return response()->json('Forbidden', 403);
        }
        // Validate data
        $data = $request->all();
        if (count($data) <= 0) {
            return response()->json('At least one field is requiered', 400);
        }
        if (!News::isValid($data)) {
            return response()->json('Bad request', 400);
        }
        if (isset($data['title'])) {
            $news->title = $data['title'];
        }
        if (isset($data['body'])) {
            $news->body = $data['body'];
        }       
        $news->save();
        return $news;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $user = User::find(1);
        if (!$user->hasRole(Role::DELETE, 'News')){
            return response()->json('Acces denied', 401);
        }
        // Validation ?
        $new = News::find($id);
        if (!isset($new)) {
            return response()->json('Not found', 404);
        }
        // Owner ?        
        $new->user;
        if ($new->user->id != $user->id) {
            return response()->json('Forbidden', 403);
        }
        $new->delete();
        return response()->json('Ok', 200);
    }
}
