<?php
namespace App\Ws\Controllers;
use Auth as LaraAuth;
use Request;
use App\Ws\RouterAuth;
use App\Models\Chat as ModelChat;

class Chat { 
    
    public function usersList(RouterAuth $router, $from)
    {        
        foreach ($router->getAuthClients() as $client) {
            $router->sendData('connection', [
                'email' => $client->user['email'],
                'id' => $client->user['id'],
            ], $from);
        }
    }
    
    public function send(RouterAuth $router, $from)
    {        
        $msg = trim(Request::input('msg', ''));  
        if (!ModelChat::isValid(['msg' => $msg, "user_id" => $from->user['id']])) {
            $router->sendData('error', 'Msg invalid', $from);           
            return;
        }        
        $chatMsg = new ModelChat([
            "msg" => $msg,
            "user_id" => $from->user['id']
        ]);
        $chatMsg->save();
        // Est-ce une commande ?
        if (mb_substr($msg, 0, 1) == '/') {
            $this->dispatchCommand($router, $from, $msg);
            return;
        }
        $router->sendDataToAllAuth('send', [
            'email' => $from->user['email'],
            'msg' => $msg,
        ]);
    }
    
    private function dispatchCommand(RouterAuth $router, $from, $cmdLine)
    {
        // Retourne la ligne de commande à l'utilisateur pour affichage
        $router->sendData('system', [            
            'msg' => $cmdLine,
        ], $from);
        // Analyse de la ligne de commande
        $cmdData = explode(" ", $cmdLine);
        $cmd = mb_substr($cmdData[0], 1);
        $cmdPara = trim(str_replace('/' . $cmd, '', $cmdLine));
        if (!method_exists(Command::class, $cmd)) {
            $router->sendData('system', ['msg'=> 'Command not found'], $from);
            return;
        }
        // Dispatcher
        call_user_func_array(
            [Command::class, $cmd],
            [$router, $from, $cmdPara]
        ); 
    }
    
}
